require "uri"
require "mime_info"

class MimeIcon
	attr_reader :file, :mime

	def initialize(file)
		@file = file.dup

		# try mime-type:subtype.png first, use
		# mime-type.png as fallback
		@mime = file.gsub(/mime-(.*):(.*).png/, "\\1/\\2") ||
		        file.gsub(/mime-(.*).png/, "\\1")
	end
end

class MimeIndex
	attr_reader :dir

	def initialize(dir)
		@dir = File.join(dir, "MIME")
		@icons = {}

		root = ENV["SCRIPT_FILENAME"][0..-(ENV["SCRIPT_NAME"].length + 1)]

		Dir.foreach(File.join(root, @dir)) do |file|
			next if file.match(/mime-.*\.png/).nil?

			icon = MimeIcon.new(file)
			@icons[icon.mime] = icon
		end
	end

	def lookup(mime)
		[
			mime,
			mime.gsub(/\/.*/, ""), # strip the subtype
			"text/plain", # fallback 1
			"application" # fallback 2
		].each do |m|
			return File.join(@dir, @icons[m].file) if @icons[m]
		end

		nil
	end
end

class IndexerEntry
	include Comparable

	attr_reader :file

	def initialize(file, icons)
		@file = file
		@basename = file

		stat = File.stat(@file)
		@mtime = stat.mtime.strftime("%Y-%m-%d %H:%M")
		@size = stat.size

		# get mime and icon
		mime = MimeInfo.instance.type(@file) || "text/plain"

		@icon = icons.lookup(mime)
	end

	def <=>(b)
		if !(directory? ^ b.directory?)
			@file <=> b.file
		elsif directory?
			-1
		elsif b.directory?
			1
		end
	end

	def directory?
		File.stat(@file).directory?
	end

	def to_s
<<EOT
	<div class="indexer_entry">
		<div>
			<a href="#{URI.escape(@basename)}">
				<img src="#{@icon}" alt=""/>
				<br/>#{truncate_basename}
			</a>
			<br/>#{@mtime}
			<br/>#{format_size}
		</div>
	</div>
EOT
	end

	private
	def format_size
		[
			[20, "MB"],
			[10, "KB"],
			[0, "B"]
		].each do |(exp, unit)|
			tmp = @size / (2 ** exp).to_f
			return "%.2f %s" % [tmp, unit] if tmp > 1
		end
	end

	def truncate_basename
		if @basename.length > 30
			@basename[0..26] + "..."
		else
			@basename
		end
	end
end

class Indexer
	def initialize(icondir, exclude)
		exclude = exclude
		icons = MimeIndex.new(icondir)

		@hits = []

		Dir["*"].each do |file|
			# ignore hidden files and stuff in @exclude
			next if file[0, 1] == "." || exclude.include?(file)

			begin
				@hits << IndexerEntry.new(file, icons)
			rescue Exception
			end
		end

		@hits.sort!
	end

	def to_s
		@hits.map { |h| h.to_s }.join
	end
end
